@@ -292,7 +292,8 @@ parse_portname(struct ipset_session *session, const char *str,
 
 error:
 	free(saved);
-	return syntax_err("cannot parse '%s' as a %s port", str, proto);
+	return ipset_warn(session, "cannot parse '%s' as a %s port",
+			  str, proto);
 }
 
 /**
@@ -313,21 +314,22 @@ ipset_parse_port(struct ipset_session *session,
 		 const char *proto)
 {
 	uint16_t port;
-	int err;
 
 	assert(session);
 	assert(opt == IPSET_OPT_PORT || opt == IPSET_OPT_PORT_TO);
 	assert(str);
 
-	if ((err = string_to_u16(session, str, &port)) == 0 ||
-	    (err = parse_portname(session, str, &port, proto)) == 0)
-		err = ipset_session_data_set(session, opt, &port);
-
-	if (!err)
-		/* No error, so reset false error messages! */
+	if (parse_portname(session, str, &port, proto) == 0) {
+		return ipset_session_data_set(session, opt, &port);
+	}
+	/* Error is stored as warning in session report */
+	if (string_to_u16(session, str, &port) == 0) {
+		/* No error, so reset false error messages */
 		ipset_session_report_reset(session);
-
-	return err;
+		return ipset_session_data_set(session, opt, &port);
+	}
+	/* Restore warning as error */
+	return ipset_session_warning_as_error(session);
 }
 
 /**
@@ -441,7 +443,7 @@ ipset_parse_tcp_port(struct ipset_session *session,
  */
 int
 ipset_parse_single_tcp_port(struct ipset_session *session,
-		     enum ipset_opt opt, const char *str)
+			    enum ipset_opt opt, const char *str)
 {
 	assert(session);
 	assert(opt == IPSET_OPT_PORT || opt == IPSET_OPT_PORT_TO);
@@ -757,7 +759,7 @@ print_warn(struct ipset_session *session)
 {
 	if (!ipset_envopt_test(session, IPSET_ENV_QUIET))
 		fprintf(stderr, "Warning: %s",
-			ipset_session_warning(session));
+			ipset_session_report_msg(session));
 	ipset_session_report_reset(session);
 }
 
@@ -1304,8 +1306,9 @@ ipset_parse_ip4_net6(struct ipset_session *session,
 		ipset_data_set(data, IPSET_OPT_FAMILY, &family);
 	}
 
-	return family == NFPROTO_IPV4 ? parse_ip(session, opt, str, IPADDR_ANY)
-				 : ipset_parse_ipnet(session, opt, str);
+	return family == NFPROTO_IPV4 ?
+		parse_ip(session, opt, str, IPADDR_ANY) :
+		ipset_parse_ipnet(session, opt, str);
 
 }
 
@@ -1332,7 +1335,7 @@ ipset_parse_timeout(struct ipset_session *session,
 	assert(opt == IPSET_OPT_TIMEOUT);
 	assert(str);
 
-	err = string_to_number_ll(session, str, 0, UINT_MAX/1000, &llnum);
+	err = string_to_number_ll(session, str, 0, (UINT_MAX>>1)/1000, &llnum);
 	if (err == 0) {
 		/* Timeout is expected to be 32bits wide, so we have
 		   to convert it here */
@@ -1394,10 +1397,11 @@ ipset_parse_iptimeout(struct ipset_session *session,
 #define check_setname(str, saved)					\
 do {									\
 	if (strlen(str) > IPSET_MAXNAMELEN - 1) {			\
-		if (saved != NULL)					\
-			free(saved);					\
-		return syntax_err("setname '%s' is longer than %u characters",\
+		int __err;						\
+		__err = syntax_err("setname '%s' is longer than %u characters",\
 				  str, IPSET_MAXNAMELEN - 1);		\
+		free(saved);						\
+		return __err;						\
 	}								\
 } while (0)
 
@@ -1537,7 +1541,7 @@ ipset_parse_before(struct ipset_session *session,
  */
 int
 ipset_parse_after(struct ipset_session *session,
-		   enum ipset_opt opt, const char *str)
+		  enum ipset_opt opt, const char *str)
 {
 	struct ipset_data *data;
 
@@ -1806,7 +1810,7 @@ ipset_parse_iface(struct ipset_session *session,
  * Returns 0 on success or a negative error code.
  */
 int ipset_parse_comment(struct ipset_session *session,
-		       enum ipset_opt opt, const char *str)
+			enum ipset_opt opt, const char *str)
 {
 	struct ipset_data *data;
 
@@ -1847,7 +1851,7 @@ ipset_parse_skbmark(struct ipset_session *session,
 					  " MARK/MASK or MARK (see manpage)");
 	}
 	result = ((uint64_t)(mark) << 32) | (mask & 0xffffffff);
-	return ipset_data_set(data, IPSET_OPT_SKBMARK, &result);
+	return ipset_data_set(data, opt, &result);
 }
 
 int
@@ -1869,35 +1873,7 @@ ipset_parse_skbprio(struct ipset_session *session,
 		return syntax_err("Invalid skbprio format, it should be:"\
 				  "MAJOR:MINOR (see manpage)");
 	major = ((uint32_t)maj << 16) | (min & 0xffff);
-	return ipset_data_set(data, IPSET_OPT_SKBPRIO, &major);
-}
-
-/**
- * ipset_parse_output - parse output format name
- * @session: session structure
- * @opt: option kind of the data
- * @str: string to parse
- *
- * Parse output format names and set session mode.
- * The value is stored in the session.
- *
- * Returns 0 on success or a negative error code.
- */
-int
-ipset_parse_output(struct ipset_session *session,
-		   int opt UNUSED, const char *str)
-{
-	assert(session);
-	assert(str);
-
-	if (STREQ(str, "plain"))
-		return ipset_session_output(session, IPSET_LIST_PLAIN);
-	else if (STREQ(str, "xml"))
-		return ipset_session_output(session, IPSET_LIST_XML);
-	else if (STREQ(str, "save"))
-		return ipset_session_output(session, IPSET_LIST_SAVE);
-
-	return syntax_err("unknown output mode '%s'", str);
+	return ipset_data_set(data, opt, &major);
 }
 
 /**
@@ -1920,8 +1896,9 @@ ipset_parse_ignored(struct ipset_session *session,
 
 	if (!ipset_data_ignored(ipset_session_data(session), opt))
 		ipset_warn(session,
-			   "Option %s is ignored. "
-			   "Please upgrade your syntax.", str);
+			   "Option '--%s %s' is ignored. "
+			   "Please upgrade your syntax.",
+			   ipset_ignored_optname(opt), str);
 
 	return 0;
 }
@@ -1941,8 +1918,8 @@ ipset_parse_ignored(struct ipset_session *session,
  */
 int
 ipset_call_parser(struct ipset_session *session,
-				  const struct ipset_arg *arg,
-				  const char *str)
+		  const struct ipset_arg *arg,
+		  const char *str)
 {
 	struct ipset_data *data = ipset_session_data(session);
 
